"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("util");
const model_builder_1 = require("./model-builder");
/**
 * This class serves as a data access object and allowes communication with the mysql server
 * It automatically strips away attributes that not match columns
 */
class TypedInserter {
    constructor(knex, dbModel) {
        this.knex = knex;
        this.dbModel = dbModel;
    }
    static getSingleTonInstance(knex, dbName) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.promise) {
                this.promise = new TypedInserter(knex).init(dbName);
            }
            return this.promise;
        });
    }
    /**
     * init a database schema if none were provided at constructor
     */
    init(dbName) {
        return __awaiter(this, void 0, void 0, function* () {
            const modelBuilder = new model_builder_1.default(this.knex, dbName);
            this.dbModel = yield modelBuilder.renderDatabaseSchema();
            return this;
        });
    }
    batchInsert(tableName, rows) {
        return __awaiter(this, void 0, void 0, function* () {
            const stripped = rows.map(row => this.stripNoneBelonging(tableName, row));
            yield this.knex.batchInsert(tableName, stripped, 1000);
        });
    }
    batchInsertIgnore(tableName, rows) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const item of rows) {
                yield this.insertIgnore(tableName, item);
            }
        });
    }
    batchUpdate(tableName, data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (data) {
                for (const item of data) {
                    yield this.update(tableName, item);
                }
            }
        });
    }
    /**
     * return true if object has its primary key among its items
     * @param tableName
     * @param data
     */
    hasPrimaryKey(tableName, data) {
        return !!(data[this.getPrimaryKey(tableName)]);
    }
    /**
     * Insert an object, if it exist, then silentely ignore and continue
     * @param tableName
     * @param item
     */
    insertIgnore(tableName, item) {
        return __awaiter(this, void 0, void 0, function* () {
            item = this.stripNoneBelonging(tableName, item);
            const qry = this.knex(tableName).insert(item).toString();
            yield this.knex.raw(qry.replace("insert", "INSERT IGNORE"));
        });
    }
    insertOnDuplicateUpdate(tableName, data) {
        return __awaiter(this, void 0, void 0, function* () {
            let filtered = this.stripNoneBelonging(tableName, data);
            const insert = this.knex(tableName).insert(filtered);
            filtered = this.removePrimaryKeys(tableName, filtered);
            const update = this.knex(tableName).update(filtered);
            const middle = "%s on duplicate key update %s";
            const lastPart = update.toString().replace(/^update `[^`]+` set /i, "");
            const query = util_1.format(middle, insert.toString(), lastPart);
            yield this.knex.raw(query);
        });
    }
    /***
    * @param tableName
    * @param object
    * Replaces all tinyint columns with a boolean
    */
    boolfix(tableName, object) {
        const table = this.dbModel.tables[tableName];
        const BOOL_TYPE = "tinyint";
        for (const key in table) {
            if (table[key].type === BOOL_TYPE) {
                object[key] = !!object[key];
            }
        }
        return object;
    }
    /**
     * @param tableName
     * @param object
     *
     * Returns the same object with all date fields
     * strings parsed into date objects
     */
    datefix(tableName, object) {
        const table = this.dbModel.tables[tableName];
        const DATE_TYPE = "date";
        for (const key in table) {
            const col = table[key];
            if (col.type === DATE_TYPE && object[key]) {
                const value = object[key];
                const isoStr = value.toISOString();
                object[key] = isoStr.substring(0, 10);
            }
        }
        return object;
    }
    /**
     * @param tableName
     * @param data
     * @param forceInsert
     * Insert an object that already has its primary key among its variables
     */
    insertWithPrimaryKey(tableName, data, forceInsert) {
        return __awaiter(this, void 0, void 0, function* () {
            data = this.stripNoneBelonging(tableName, data);
            if (Object.keys(data).length > 1 || forceInsert) {
                yield this.knex(tableName).insert(data);
            }
            return data;
        });
    }
    /**
     * returns true if the first primary key column is of type int
     * @param tableName
     */
    isIntPk(tableName) {
        const pks = this.getPkCols(tableName);
        if (pks.length > 0) {
            return pks[0].type === "int";
        }
        return false;
    }
    /**
     * Returns the name of the first primary key column
     * @param tableName
     */
    getPrimaryKey(tableName) {
        const cols = this.getPkCols(tableName);
        return (cols == null || cols.length === 0) ? null : cols[0].field;
    }
    /**
     * @param tableName
     * @param data
     * Return a copy of the object without any primary keys
     */
    removePrimaryKeys(tableName, data) {
        const filtered = JSON.parse(JSON.stringify(data)); // copy
        this.getPrimaryKeyNames(tableName).forEach(pk => delete filtered[pk]);
        return filtered;
    }
    /**
     * Insert but if the object exists throw the other one away.
     * @param tableName
     * @param item
     */
    replaceInto(tableName, item) {
        return __awaiter(this, void 0, void 0, function* () {
            item = this.stripNoneBelonging(tableName, item);
            const q = this.knex(tableName).insert(item).toString();
            yield this.knex.raw(q.replace("insert", "replace"));
        });
    }
    /**
     * Update or insert a data objet.
     * Update if object has a primary key already.
     * To insert an object with a primary key, use the other function
     * @param tableName
     * @param data
     */
    setData(tableName, data) {
        return __awaiter(this, void 0, void 0, function* () {
            data = this.stripNoneBelonging(tableName, data);
            if (this.hasPrimaryKey(tableName, data)) {
                // update existing
                const pk = this.getPrimaryKey(tableName);
                const pkValue = data[pk];
                yield this.update(tableName, data);
                data[pk] = pkValue;
                return data;
            }
            else {
                // new object, insert
                return this.insert(tableName, data);
            }
        });
    }
    forceUpdate(tableName, data) {
        return __awaiter(this, void 0, void 0, function* () {
            data = this.stripNoneBelonging(tableName, data);
            return yield this.update(tableName, data);
        });
    }
    /**
     * return a new object where with only attributes that have
     * a corresponding column for given table
     * @param table
     * @param data
     */
    stripNoneBelonging(tableName, data) {
        const table = this.dbModel.tables[tableName];
        const copy = {};
        for (const key in data) {
            if (table[key]) {
                copy[key] = data[key];
            }
        }
        return copy;
    }
    update(tableName, data) {
        return __awaiter(this, void 0, void 0, function* () {
            const pks = this.getPrimaryKeyNames(tableName);
            const criterions = {};
            for (const pk of pks) {
                criterions[pk] = data[pk];
                delete data[pk];
            }
            if (Object.keys(data).length > 0) {
                yield this.knex(tableName).where(criterions).update(data);
            }
            return data;
        });
    }
    insert(tableName, data) {
        return __awaiter(this, void 0, void 0, function* () {
            const reply = yield this.knex(tableName).insert(data);
            data[this.getPrimaryKey(tableName)] = reply[0];
            return data;
        });
    }
    getPkCols(tableName) {
        if (!tableName) {
            return null;
        }
        const cols = [];
        const table = this.dbModel.tables[tableName];
        for (const key in table) {
            if (table[key].isPrimary) {
                cols.push(table[key]);
            }
        }
        return cols;
    }
    /**
     * List the names of all primary keys in a table
     * @param tableName
     */
    getPrimaryKeyNames(tableName) {
        const cols = this.getPkCols(tableName);
        if (!cols || cols.length === 0) {
            return null;
        }
        return cols.map(col => col.field);
    }
}
exports.default = TypedInserter;
//# sourceMappingURL=updater.js.map